#!/usr/bin/python3

class VTCommandLineArgsBase:
    def __init__(self, metadata_file, execution_url, debug, output_filename, api_access_key, insecure, cert):
        self.metadata_file = metadata_file
        self.execution_url = execution_url
        self.debug = debug
        self.output_filename = output_filename
        self.api_access_key = api_access_key
        self.insecure = insecure
        self.cert = cert



#!/usr/bin/python3
import argparse
import json
import os

from google.protobuf.timestamp_pb2 import Timestamp
from common.api import virtual_sfv_pb2_grpc
from common.api import virtual_sfv_pb2
from common.api import solar_field_common_pb2
from common.api import common_models_pb2
from common import utils
import grpc
from datetime import datetime, timedelta
from common.models import VTCommandLineArgsBase
TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
import pandas as pd
from google.protobuf import wrappers_pb2
import google.protobuf.json_format as json_format
from typing import List


def build_query(query_time_utc: datetime, margin: timedelta, defined_inputs: List[common_models_pb2.VirtualModuleInputTag],  calculation_method: solar_field_common_pb2.SfvCalculationMethod) -> virtual_sfv_pb2.SfvQuery:
    res = virtual_sfv_pb2.SfvQuery(calculation_method=calculation_method)
    for input in defined_inputs:
        res.tags_descriptions.append(input.raw_id)
    res.query_time_utc.FromDatetime(query_time_utc)
    res.margin.FromTimedelta(margin)
    return res

def get_data(query: virtual_sfv_pb2.SfvQuery, args: VTCommandLineArgsBase) -> solar_field_common_pb2.SolarFieldDto:
    channel = utils.get_grpc_channel_according_to_args(args)
    client = virtual_sfv_pb2_grpc.VirtualSfvGrpcServiceStub(channel)
    res = client.GetData(query, metadata=[('x-api-key', f'{args.api_access_key}')])
    problematics = res.problematic_tags
    if len(problematics) > 0:
        raise Exception(f"Didn't get data for {len(problematics)} tags, reasons: {problematics}")
    return res


def proto_float_to_number(float_value:wrappers_pb2.FloatValue) -> float:
    return float_value.value

def solar_field_to_df(defined_inputs: List[common_models_pb2.VirtualModuleInputTag], sf_dto: solar_field_common_pb2.SolarFieldDto = None) -> pd.DataFrame:
    res: pd.DataFrame = pd.DataFrame()
    if len(defined_inputs) == 0:
        return res
    problematics = sf_dto.problematic_tags
    if len(problematics) > 0:
        raise Exception(
            f"Didn't get data for {len(problematics)} tags, reasons: {json_format.MessageToJson(sf_dto)}")
    if (sf_dto == None):
        return res
    for tag_data in sf_dto.data:
        tag_variable = ""
        for input_tag in defined_inputs:
            if input_tag.raw_id == tag_data.tag_id:
                tag_variable = input_tag.variable_name
                break
        values_list = []
        for float_nullable_value in enumerate(tag_data.nullable_data):
            values_list.append(float_nullable_value[1])
        res[tag_variable] = values_list
        res[tag_variable] = res[tag_variable].apply(proto_float_to_number)
    return res

def plot_series(series: pd.Series, helio_radius=0.05, color_map: str = 'turbo'):
    # Get layout and shift
    layout_df = pd.read_csv("HeliostatsLayout.csv").set_index("Id")
    import matplotlib.pyplot as plt
    rdf = pd.Series([0 for i in range(layout_df.index.size)])
    for idx, item in enumerate(series):
        rdf[idx] = item
    im = plt.scatter(x =layout_df["E"], y = layout_df["N"],c=rdf,s = helio_radius,  cmap=plt.cm.get_cmap(color_map, 255))
    plt.colorbar(im)
    plt.show()

def solar_field_to_update_time(defined_inputs: List[common_models_pb2.VirtualModuleInputTag], sf_dto: solar_field_common_pb2.SolarFieldDto = None) -> pd.DataFrame:
    res: pd.DataFrame = pd.DataFrame()
    if len(defined_inputs) == 0:
        return res
    problematics = sf_dto.problematic_tags
    if len(problematics) > 0:
        raise Exception(
            f"Didn't get data for {len(problematics)} tags, reasons: {json_format.MessageToJson(sf_dto)}")
    if (sf_dto == None):
        return res
    times = []
    for tag_data in sf_dto.data:
        tag_variable = ""
        for input_tag in defined_inputs:
            if input_tag.raw_id == tag_data.tag_id:
                tag_variable = input_tag.variable_name
                break
        times.append(tag_data.update_time.ToMilliseconds())
    res[tag_variable] = times
    return res


def parse_input(cmd_line_args) -> VTCommandLineArgsBase:
    (base_cmd_line_args, extra_args) = utils.parse_input(cmd_line_args)
    parser = argparse.ArgumentParser(
        description='Input parameters to run virtual sfv')
    parser.add_argument('--query_time_utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='query_time_utc',
                        help='The query time in UTC, format YYYY-MM-DD hh:mm:ss')
    parser.add_argument('--margin', action='store', required='--metadata' not in cmd_line_args,
                        dest='margin',
                        help='The margin time of the query in UTC, format hh:mm:ss')
    args = parser.parse_args(extra_args)
    if args.query_time_utc is not None and args.margin is not None:
        query_time_utc = datetime.strptime(args.query_time_utc, TIME_FORMAT)
        margin = utils.parse_time(args.margin)
    else:
        query_time_utc = None
        margin = None
    return (base_cmd_line_args, query_time_utc, margin)



#!/usr/bin/python3
import os,sys
currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)
from datetime import datetime, timedelta
import pandas as pd
from sys import argv
from sf_virtual_data.virtual_table import solar_focus_tables_lib as sofoc
from sf_virtual_data.virtual_table.virtual_tables_runner import VirtualTablesRunner
from sf_virtual_data.common.api import virtual_sfv_pb2
from sf_virtual_data.common.api import solar_field_common_pb2
import zipfile

def main(cmd_line_args=argv[1:]) -> pd.DataFrame:
    base_cmd_line_args, sfv_query_time_utc, sfv_margin, trends_start_utc, trends_end_utc = sofoc.parse_input(cmd_line_args)
    vt = VirtualTablesRunner( base_cmd_line_args.metadata_file)
    res = vt.execute(base_cmd_line_args, sfv_query_time_utc, sfv_margin, trends_start_utc, trends_end_utc)
    if not base_cmd_line_args.debug:
        with zipfile.ZipFile(base_cmd_line_args.output_filename, 'w') as myzip:
            for name, table in res.items():
                csv_name = name + ".csv"
                table.to_csv(csv_name)
                myzip.write(csv_name)
    return res


if __name__ == "__main__":
    """
    Example of script run:  python virtual_tables_entry_point.py --query_time_utc "2020-10-07 02:00:00" --margin "00:01:00" --debug --api-access-key "<your_api_access_key>"
    or run with IDE run-time args  ["--query_time_utc","2020-10-07 02:00:00", "--margin", "00:01:00", "--debug", "--api-access-key", "<your_api_access_key>"]

    """
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: virtual-tables.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from . import virtual_trends_pb2 as Protobuf_dot_virtual__trends__pb2
from . import virtual_sfv_pb2 as Protobuf_dot_virtual__sfv__pb2
from . import solar_field_common_pb2 as Protobuf_dot_solar__field__common__pb2
from . import common_models_pb2 as Protobuf_dot_common__models__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='virtual-tables.proto',
  package='Bsii.SolarFocus.Protobuf',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14virtual-tables.proto\x12\x18\x42sii.SolarFocus.Protobuf\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dProtobuf/virtual-trends.proto\x1a\x1aProtobuf/virtual-sfv.proto\x1a!Protobuf/solar-field-common.proto\x1a\x1cProtobuf/common-models.proto\"\xa3\x01\n\x12VirtualTablesQuery\x12K\n\x0ctrends_query\x18\x01 \x01(\x0b\x32\x35.Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryInput\x12@\n\tsfv_query\x18\x02 \x01(\x0b\x32-.Bsii.SolarFocus.Protobuf.SolarField.SfvQuery\"$\n\x14VirtualTableMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\"\xb1\x04\n\x13VirtualTablesModule\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x12\n\nbase_image\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\x13\n\x0b\x65nvironment\x18\x05 \x01(\t\x12Y\n\x16sfv_calculation_method\x18\x06 \x01(\x0e\x32\x39.Bsii.SolarFocus.Protobuf.SolarField.SfvCalculationMethod\x12U\n\x19trends_calculation_method\x18\x07 \x01(\x0e\x32\x32.Bsii.SolarFocus.Protobuf.Trends.CalculationMethod\x12\x33\n\x10sample_frequency\x18\x08 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x45\n\x0cinput_trends\x18\t \x03(\x0b\x32/.Bsii.SolarFocus.Protobuf.VirtualModuleInputTag\x12G\n\x0einput_sfv_tags\x18\n \x03(\x0b\x32/.Bsii.SolarFocus.Protobuf.VirtualModuleInputTag\x12\x45\n\routput_tables\x18\x0b \x03(\x0b\x32..Bsii.SolarFocus.Protobuf.VirtualTableMetadata\"\xa4\x01\n\x19VirtualTablesInputsResult\x12?\n\x03sfv\x18\x01 \x01(\x0b\x32\x32.Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto\x12\x46\n\x06trends\x18\x02 \x01(\x0b\x32\x36.Bsii.SolarFocus.Protobuf.Trends.TimeSeriesQueryResult*=\n\x0cResultStatus\x12\x0b\n\x07SUCCESS\x10\x00\x12\x13\n\x0fPARTIAL_SUCCESS\x10\x01\x12\x0b\n\x07\x46\x41ILURE\x10\x02\x32\x88\x01\n\x18VirtualTablesGrpcService\x12l\n\x07GetData\x12,.Bsii.SolarFocus.Protobuf.VirtualTablesQuery\x1a\x33.Bsii.SolarFocus.Protobuf.VirtualTablesInputsResultB\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,Protobuf_dot_virtual__trends__pb2.DESCRIPTOR,Protobuf_dot_virtual__sfv__pb2.DESCRIPTOR,Protobuf_dot_solar__field__common__pb2.DESCRIPTOR,Protobuf_dot_common__models__pb2.DESCRIPTOR,])

_RESULTSTATUS = _descriptor.EnumDescriptor(
  name='ResultStatus',
  full_name='Bsii.SolarFocus.Protobuf.ResultStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARTIAL_SUCCESS', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FAILURE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1141,
  serialized_end=1202,
)
_sym_db.RegisterEnumDescriptor(_RESULTSTATUS)

ResultStatus = enum_type_wrapper.EnumTypeWrapper(_RESULTSTATUS)
SUCCESS = 0
PARTIAL_SUCCESS = 1
FAILURE = 2



_VIRTUALTABLESQUERY = _descriptor.Descriptor(
  name='VirtualTablesQuery',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTablesQuery',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trends_query', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesQuery.trends_query', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sfv_query', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesQuery.sfv_query', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=207,
  serialized_end=370,
)


_VIRTUALTABLEMETADATA = _descriptor.Descriptor(
  name='VirtualTableMetadata',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTableMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.VirtualTableMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=372,
  serialized_end=408,
)


_VIRTUALTABLESMODULE = _descriptor.Descriptor(
  name='VirtualTablesModule',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='username', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_image', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.base_image', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.environment', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sfv_calculation_method', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.sfv_calculation_method', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trends_calculation_method', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.trends_calculation_method', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_frequency', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.sample_frequency', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_trends', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.input_trends', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='input_sfv_tags', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.input_sfv_tags', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_tables', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesModule.output_tables', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=411,
  serialized_end=972,
)


_VIRTUALTABLESINPUTSRESULT = _descriptor.Descriptor(
  name='VirtualTablesInputsResult',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTablesInputsResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sfv', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesInputsResult.sfv', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='trends', full_name='Bsii.SolarFocus.Protobuf.VirtualTablesInputsResult.trends', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=975,
  serialized_end=1139,
)

_VIRTUALTABLESQUERY.fields_by_name['trends_query'].message_type = Protobuf_dot_virtual__trends__pb2._TIMESERIESQUERYINPUT
_VIRTUALTABLESQUERY.fields_by_name['sfv_query'].message_type = Protobuf_dot_virtual__sfv__pb2._SFVQUERY
_VIRTUALTABLESMODULE.fields_by_name['sfv_calculation_method'].enum_type = Protobuf_dot_solar__field__common__pb2._SFVCALCULATIONMETHOD
_VIRTUALTABLESMODULE.fields_by_name['trends_calculation_method'].enum_type = Protobuf_dot_virtual__trends__pb2._CALCULATIONMETHOD
_VIRTUALTABLESMODULE.fields_by_name['sample_frequency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_VIRTUALTABLESMODULE.fields_by_name['input_trends'].message_type = Protobuf_dot_common__models__pb2._VIRTUALMODULEINPUTTAG
_VIRTUALTABLESMODULE.fields_by_name['input_sfv_tags'].message_type = Protobuf_dot_common__models__pb2._VIRTUALMODULEINPUTTAG
_VIRTUALTABLESMODULE.fields_by_name['output_tables'].message_type = _VIRTUALTABLEMETADATA
_VIRTUALTABLESINPUTSRESULT.fields_by_name['sfv'].message_type = Protobuf_dot_solar__field__common__pb2._SOLARFIELDDTO
_VIRTUALTABLESINPUTSRESULT.fields_by_name['trends'].message_type = Protobuf_dot_virtual__trends__pb2._TIMESERIESQUERYRESULT
DESCRIPTOR.message_types_by_name['VirtualTablesQuery'] = _VIRTUALTABLESQUERY
DESCRIPTOR.message_types_by_name['VirtualTableMetadata'] = _VIRTUALTABLEMETADATA
DESCRIPTOR.message_types_by_name['VirtualTablesModule'] = _VIRTUALTABLESMODULE
DESCRIPTOR.message_types_by_name['VirtualTablesInputsResult'] = _VIRTUALTABLESINPUTSRESULT
DESCRIPTOR.enum_types_by_name['ResultStatus'] = _RESULTSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VirtualTablesQuery = _reflection.GeneratedProtocolMessageType('VirtualTablesQuery', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTABLESQUERY,
  '__module__' : 'virtual_tables_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualTablesQuery)
  })
_sym_db.RegisterMessage(VirtualTablesQuery)

VirtualTableMetadata = _reflection.GeneratedProtocolMessageType('VirtualTableMetadata', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTABLEMETADATA,
  '__module__' : 'virtual_tables_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualTableMetadata)
  })
_sym_db.RegisterMessage(VirtualTableMetadata)

VirtualTablesModule = _reflection.GeneratedProtocolMessageType('VirtualTablesModule', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTABLESMODULE,
  '__module__' : 'virtual_tables_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualTablesModule)
  })
_sym_db.RegisterMessage(VirtualTablesModule)

VirtualTablesInputsResult = _reflection.GeneratedProtocolMessageType('VirtualTablesInputsResult', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTABLESINPUTSRESULT,
  '__module__' : 'virtual_tables_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualTablesInputsResult)
  })
_sym_db.RegisterMessage(VirtualTablesInputsResult)


DESCRIPTOR._options = None

_VIRTUALTABLESGRPCSERVICE = _descriptor.ServiceDescriptor(
  name='VirtualTablesGrpcService',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTablesGrpcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1205,
  serialized_end=1341,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetData',
    full_name='Bsii.SolarFocus.Protobuf.VirtualTablesGrpcService.GetData',
    index=0,
    containing_service=None,
    input_type=_VIRTUALTABLESQUERY,
    output_type=_VIRTUALTABLESINPUTSRESULT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_VIRTUALTABLESGRPCSERVICE)

DESCRIPTOR.services_by_name['VirtualTablesGrpcService'] = _VIRTUALTABLESGRPCSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
import pandas as pd
from datetime import datetime, timedelta

def calculate(data: pd.DataFrame, start_utc: datetime, end_utc: datetime) -> pd.DataFrame:
    """
    User main code to calculate new virtual trends based on other trends

    Parameters
    ----------
    data: pd.DataFrame
        pandas DataFrame. columns are based on define_inputs()

    Returns
    -------
        pd.DataFrame
        pandas DataFrame. column must match the defined output as implemented in define_outputs() function
    """

    ################# User Code #########################

    # This returns output for 1 tag "HlstShortStatus.currentState" from Mongo collection
    res = read_from_mongo(start_utc, end_utc, timedelta(minutes= 5), "HlstShortStatus.currentState")

    # This returns output for 1 tag "OkCount" from Sql Server (CALS table)
    res = read_from_sql_server(start_utc, end_utc)

    ################# End of User Code ###################
    return res


def read_from_sql_server(start_utc: datetime, end_etc: datetime) -> pd.DataFrame:
    from .sql_reader import SqlReader
    date_format_to_sql_server = "%Y-%m-%d"
    query = '''SELECT T.MeasureTime  , Count(*) as OkCount FROM
                (SELECT heliostatId ,CONVERT(varchar, MeasureTime, 103) as MeasureTime
                FROM [NoorPA_SfincsData].[CMS_CALS].[Measurements]
                WHERE Status = 0
                and MeasureTime BETWEEN '{0}' AND '{1}'
                GROUP BY heliostatId , MeasureTime) T
            GROUP BY T.MeasureTime
            ORDER BY T.MeasureTime DESC
    '''.format(start_utc.strftime(date_format_to_sql_server), end_etc.strftime(date_format_to_sql_server))

    reader = SqlReader(SqlReader.create_sql_server_trusted_connection_string("site-sql-data", "NoorPA_SfincsData"))
    date_format_from_sql_server = "%d/%m/%Y"
    df = reader.get_df_from_db(query, "MeasureTime", date_format=date_format_from_sql_server)
    return df


def read_from_mongo(start_utc: datetime, end_utc: datetime, interval: timedelta, collection_name: str):
    from .mongo_reader import MongoReader
    reader = MongoReader("ConnectionString")
    query_start_time = start_utc
    ret = pd.DataFrame(columns=[collection_name])
    while (query_start_time < end_utc):
        query_end_time = query_start_time + interval
        if (query_end_time > end_utc):
            query_end_time = end_utc
        print("getting collections between {0} and {1}".format(query_start_time, {query_end_time}))
        df = reader.get_document_as_series(collection_name,  {'_id': {'$gt': query_start_time, '$lt': query_end_time}}, [("_id", -1)], "_id", "Data")
        num_of_sleep = df[df["value"]==5].count()["value"]
        ret.loc[query_start_time] = num_of_sleep
        query_start_time = query_start_time + interval
    return ret



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: common-models.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='common-models.proto',
  package='Bsii.SolarFocus.Protobuf',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13\x63ommon-models.proto\x12\x18\x42sii.SolarFocus.Protobuf\"\xa6\x01\n\x15VirtualModuleInputTag\x12<\n\x06raw_id\x18\x01 \x01(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\x12\x10\n\x08tag_name\x18\x02 \x01(\t\x12\x13\n\x0bsource_name\x18\x03 \x01(\t\x12\x11\n\tsource_id\x18\x04 \x01(\x05\x12\x15\n\rvariable_name\x18\x05 \x01(\t\"\xbb\x02\n\x12TagsIdentifierBase\x12;\n\x08tag_type\x18\x01 \x01(\x0e\x32).Bsii.SolarFocus.Protobuf.IdentifierTypes\x12\x41\n\x0etag_identifier\x18\x02 \x01(\x0b\x32\'.Bsii.SolarFocus.Protobuf.TagIdentifierH\x00\x12P\n\x16virtual_tag_identifier\x18\x03 \x01(\x0b\x32..Bsii.SolarFocus.Protobuf.VirtualTagIdentifierH\x00\x12H\n\x12sfo_tag_identifier\x18\x04 \x01(\x0b\x32*.Bsii.SolarFocus.Protobuf.SfoTagIdentifierH\x00\x42\t\n\x07subtype\"\x1b\n\rTagIdentifier\x12\n\n\x02id\x18\x01 \x01(\x05\"2\n\x10SfoTagIdentifier\x12\x0e\n\x06tag_id\x18\x01 \x01(\x05\x12\x0e\n\x06sfo_id\x18\x02 \x01(\x05\"@\n\x14VirtualTagIdentifier\x12\x13\n\x0bmodule_name\x18\x01 \x01(\t\x12\x13\n\x0boutput_name\x18\x02 \x01(\t\"^\n\x0eProblematicTag\x12<\n\x06tag_id\x18\x01 \x01(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\x12\x0e\n\x06reason\x18\x02 \x01(\t*J\n\x0fIdentifierTypes\x12\x08\n\x04ZERO\x10\x00\x12\x0b\n\x07NUMERIC\x10\x01\x12\x13\n\x0fNUMERIC_PER_SFO\x10\x02\x12\x0b\n\x07VIRTUAL\x10\x03\x42\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
)

_IDENTIFIERTYPES = _descriptor.EnumDescriptor(
  name='IdentifierTypes',
  full_name='Bsii.SolarFocus.Protobuf.IdentifierTypes',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ZERO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMERIC', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NUMERIC_PER_SFO', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VIRTUAL', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=779,
  serialized_end=853,
)
_sym_db.RegisterEnumDescriptor(_IDENTIFIERTYPES)

IdentifierTypes = enum_type_wrapper.EnumTypeWrapper(_IDENTIFIERTYPES)
ZERO = 0
NUMERIC = 1
NUMERIC_PER_SFO = 2
VIRTUAL = 3



_VIRTUALMODULEINPUTTAG = _descriptor.Descriptor(
  name='VirtualModuleInputTag',
  full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw_id', full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag.raw_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag_name', full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag.tag_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_name', full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag.source_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_id', full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag.source_id', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variable_name', full_name='Bsii.SolarFocus.Protobuf.VirtualModuleInputTag.variable_name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=50,
  serialized_end=216,
)


_TAGSIDENTIFIERBASE = _descriptor.Descriptor(
  name='TagsIdentifierBase',
  full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_type', full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase.tag_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tag_identifier', full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase.tag_identifier', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='virtual_tag_identifier', full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase.virtual_tag_identifier', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sfo_tag_identifier', full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase.sfo_tag_identifier', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='subtype', full_name='Bsii.SolarFocus.Protobuf.TagsIdentifierBase.subtype',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=219,
  serialized_end=534,
)


_TAGIDENTIFIER = _descriptor.Descriptor(
  name='TagIdentifier',
  full_name='Bsii.SolarFocus.Protobuf.TagIdentifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Bsii.SolarFocus.Protobuf.TagIdentifier.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=536,
  serialized_end=563,
)


_SFOTAGIDENTIFIER = _descriptor.Descriptor(
  name='SfoTagIdentifier',
  full_name='Bsii.SolarFocus.Protobuf.SfoTagIdentifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_id', full_name='Bsii.SolarFocus.Protobuf.SfoTagIdentifier.tag_id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sfo_id', full_name='Bsii.SolarFocus.Protobuf.SfoTagIdentifier.sfo_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=565,
  serialized_end=615,
)


_VIRTUALTAGIDENTIFIER = _descriptor.Descriptor(
  name='VirtualTagIdentifier',
  full_name='Bsii.SolarFocus.Protobuf.VirtualTagIdentifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='module_name', full_name='Bsii.SolarFocus.Protobuf.VirtualTagIdentifier.module_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='output_name', full_name='Bsii.SolarFocus.Protobuf.VirtualTagIdentifier.output_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=617,
  serialized_end=681,
)


_PROBLEMATICTAG = _descriptor.Descriptor(
  name='ProblematicTag',
  full_name='Bsii.SolarFocus.Protobuf.ProblematicTag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_id', full_name='Bsii.SolarFocus.Protobuf.ProblematicTag.tag_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='reason', full_name='Bsii.SolarFocus.Protobuf.ProblematicTag.reason', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=683,
  serialized_end=777,
)

_VIRTUALMODULEINPUTTAG.fields_by_name['raw_id'].message_type = _TAGSIDENTIFIERBASE
_TAGSIDENTIFIERBASE.fields_by_name['tag_type'].enum_type = _IDENTIFIERTYPES
_TAGSIDENTIFIERBASE.fields_by_name['tag_identifier'].message_type = _TAGIDENTIFIER
_TAGSIDENTIFIERBASE.fields_by_name['virtual_tag_identifier'].message_type = _VIRTUALTAGIDENTIFIER
_TAGSIDENTIFIERBASE.fields_by_name['sfo_tag_identifier'].message_type = _SFOTAGIDENTIFIER
_TAGSIDENTIFIERBASE.oneofs_by_name['subtype'].fields.append(
  _TAGSIDENTIFIERBASE.fields_by_name['tag_identifier'])
_TAGSIDENTIFIERBASE.fields_by_name['tag_identifier'].containing_oneof = _TAGSIDENTIFIERBASE.oneofs_by_name['subtype']
_TAGSIDENTIFIERBASE.oneofs_by_name['subtype'].fields.append(
  _TAGSIDENTIFIERBASE.fields_by_name['virtual_tag_identifier'])
_TAGSIDENTIFIERBASE.fields_by_name['virtual_tag_identifier'].containing_oneof = _TAGSIDENTIFIERBASE.oneofs_by_name['subtype']
_TAGSIDENTIFIERBASE.oneofs_by_name['subtype'].fields.append(
  _TAGSIDENTIFIERBASE.fields_by_name['sfo_tag_identifier'])
_TAGSIDENTIFIERBASE.fields_by_name['sfo_tag_identifier'].containing_oneof = _TAGSIDENTIFIERBASE.oneofs_by_name['subtype']
_PROBLEMATICTAG.fields_by_name['tag_id'].message_type = _TAGSIDENTIFIERBASE
DESCRIPTOR.message_types_by_name['VirtualModuleInputTag'] = _VIRTUALMODULEINPUTTAG
DESCRIPTOR.message_types_by_name['TagsIdentifierBase'] = _TAGSIDENTIFIERBASE
DESCRIPTOR.message_types_by_name['TagIdentifier'] = _TAGIDENTIFIER
DESCRIPTOR.message_types_by_name['SfoTagIdentifier'] = _SFOTAGIDENTIFIER
DESCRIPTOR.message_types_by_name['VirtualTagIdentifier'] = _VIRTUALTAGIDENTIFIER
DESCRIPTOR.message_types_by_name['ProblematicTag'] = _PROBLEMATICTAG
DESCRIPTOR.enum_types_by_name['IdentifierTypes'] = _IDENTIFIERTYPES
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VirtualModuleInputTag = _reflection.GeneratedProtocolMessageType('VirtualModuleInputTag', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALMODULEINPUTTAG,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualModuleInputTag)
  })
_sym_db.RegisterMessage(VirtualModuleInputTag)

TagsIdentifierBase = _reflection.GeneratedProtocolMessageType('TagsIdentifierBase', (_message.Message,), {
  'DESCRIPTOR' : _TAGSIDENTIFIERBASE,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsIdentifierBase)
  })
_sym_db.RegisterMessage(TagsIdentifierBase)

TagIdentifier = _reflection.GeneratedProtocolMessageType('TagIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _TAGIDENTIFIER,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagIdentifier)
  })
_sym_db.RegisterMessage(TagIdentifier)

SfoTagIdentifier = _reflection.GeneratedProtocolMessageType('SfoTagIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _SFOTAGIDENTIFIER,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SfoTagIdentifier)
  })
_sym_db.RegisterMessage(SfoTagIdentifier)

VirtualTagIdentifier = _reflection.GeneratedProtocolMessageType('VirtualTagIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _VIRTUALTAGIDENTIFIER,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.VirtualTagIdentifier)
  })
_sym_db.RegisterMessage(VirtualTagIdentifier)

ProblematicTag = _reflection.GeneratedProtocolMessageType('ProblematicTag', (_message.Message,), {
  'DESCRIPTOR' : _PROBLEMATICTAG,
  '__module__' : 'common_models_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.ProblematicTag)
  })
_sym_db.RegisterMessage(ProblematicTag)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: solar-field.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from . import common_models_pb2 as Protobuf_dot_common__models__pb2
from . import solar_field_common_pb2 as Protobuf_dot_solar__field__common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='solar-field.proto',
  package='Bsii.SolarFocus.Protobuf.SolarField',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11solar-field.proto\x12#Bsii.SolarFocus.Protobuf.SolarField\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cProtobuf/common-models.proto\x1a!Protobuf/solar-field-common.proto\"\xfc\x02\n\x14StreamSolarFieldArgs\x12\x32\n\x0estart_time_utc\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0c\x65nd_time_utc\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\x10sample_frequency\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12)\n\x06margin\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12U\n\x12\x63\x61lculation_method\x18\x05 \x01(\x0e\x32\x39.Bsii.SolarFocus.Protobuf.SolarField.SfvCalculationMethod\x12G\n\x11tags_descriptions\x18\x06 \x03(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase2\x92\x01\n\nSolarField\x12\x83\x01\n\x10StreamSolarField\x12\x39.Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs\x1a\x32.Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto0\x01\x42\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,Protobuf_dot_common__models__pb2.DESCRIPTOR,Protobuf_dot_solar__field__common__pb2.DESCRIPTOR,])




_STREAMSOLARFIELDARGS = _descriptor.Descriptor(
  name='StreamSolarFieldArgs',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start_time_utc', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.start_time_utc', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_time_utc', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.end_time_utc', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sample_frequency', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.sample_frequency', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='margin', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.margin', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_method', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.calculation_method', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags_descriptions', full_name='Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs.tags_descriptions', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=189,
  serialized_end=569,
)

_STREAMSOLARFIELDARGS.fields_by_name['start_time_utc'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMSOLARFIELDARGS.fields_by_name['end_time_utc'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMSOLARFIELDARGS.fields_by_name['sample_frequency'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_STREAMSOLARFIELDARGS.fields_by_name['margin'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_STREAMSOLARFIELDARGS.fields_by_name['calculation_method'].enum_type = Protobuf_dot_solar__field__common__pb2._SFVCALCULATIONMETHOD
_STREAMSOLARFIELDARGS.fields_by_name['tags_descriptions'].message_type = Protobuf_dot_common__models__pb2._TAGSIDENTIFIERBASE
DESCRIPTOR.message_types_by_name['StreamSolarFieldArgs'] = _STREAMSOLARFIELDARGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StreamSolarFieldArgs = _reflection.GeneratedProtocolMessageType('StreamSolarFieldArgs', (_message.Message,), {
  'DESCRIPTOR' : _STREAMSOLARFIELDARGS,
  '__module__' : 'solar_field_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.StreamSolarFieldArgs)
  })
_sym_db.RegisterMessage(StreamSolarFieldArgs)


DESCRIPTOR._options = None

_SOLARFIELD = _descriptor.ServiceDescriptor(
  name='SolarField',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.SolarField',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=572,
  serialized_end=718,
  methods=[
  _descriptor.MethodDescriptor(
    name='StreamSolarField',
    full_name='Bsii.SolarFocus.Protobuf.SolarField.SolarField.StreamSolarField',
    index=0,
    containing_service=None,
    input_type=_STREAMSOLARFIELDARGS,
    output_type=Protobuf_dot_solar__field__common__pb2._SOLARFIELDDTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SOLARFIELD)

DESCRIPTOR.services_by_name['SolarField'] = _SOLARFIELD

# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tags-metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from . import common_models_pb2 as Protobuf_dot_common__models__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tags-metadata.proto',
  package='Bsii.SolarFocus.Protobuf.TagsMetadata',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13tags-metadata.proto\x12%Bsii.SolarFocus.Protobuf.TagsMetadata\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cProtobuf/common-models.proto\"-\n\x0e\x45numMappingDto\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05\"\xb2\x04\n\x06TagDto\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x14\n\x0c\x64isplay_name\x18\x02 \x01(\t\x12\x31\n\x0b\x64\x65scription\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12.\n\tmin_value\x18\x04 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12.\n\tmax_value\x18\x05 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12*\n\x04name\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x11\n\tsource_id\x18\x07 \x01(\x05\x12*\n\x04unit\x18\x08 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x39\n\x13raw_enum_definition\x18\t \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\x43\n\x06status\x18\n \x01(\x0e\x32\x33.Bsii.SolarFocus.Protobuf.TagsMetadata.EntityStatus\x12T\n\x0c\x65num_mapping\x18\x0b \x03(\x0b\x32>.Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.EnumMappingEntry\x1a\x32\n\x10\x45numMappingEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x80\x01\n\rDataSourceDto\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x42\n\x0fidentifier_type\x18\x03 \x01(\x0e\x32).Bsii.SolarFocus.Protobuf.IdentifierTypes\x12\x11\n\tsite_name\x18\x04 \x01(\t\"|\n\x0cUnitMetadata\x12\x0c\n\x04Unit\x18\x01 \x01(\t\x12.\n\tmin_value\x18\x02 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\x12.\n\tmax_value\x18\x03 \x01(\x0b\x32\x1b.google.protobuf.FloatValue\"\xe1\x01\n\x0fTagsMetadataDto\x12I\n\x0b\x44\x61taSources\x18\x01 \x03(\x0b\x32\x34.Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto\x12;\n\x04Tags\x18\x02 \x03(\x0b\x32-.Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto\x12\x46\n\tUnitTypes\x18\x03 \x03(\x0b\x32\x33.Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata*B\n\x0c\x45ntityStatus\x12\x08\n\x04ZERO\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0b\n\x07\x44\x45LETED\x10\x02\x12\x0f\n\x0b\x44ISAPPEARED\x10\x03\x32\xbd\x03\n\x17TagsMetadataGrpcService\x12j\n\x18GetUnfilteredAllMetaData\x12\x16.google.protobuf.Empty\x1a\x36.Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto\x12q\n\x19GetUnfilteredSiteMetaData\x12\x1c.google.protobuf.StringValue\x1a\x36.Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto\x12g\n\x0fGetSiteMetaData\x12\x1c.google.protobuf.StringValue\x1a\x36.Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto\x12Z\n\x11UpdateTagMetaData\x12-.Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto\x1a\x16.google.protobuf.EmptyB\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,Protobuf_dot_common__models__pb2.DESCRIPTOR,])

_ENTITYSTATUS = _descriptor.EnumDescriptor(
  name='EntityStatus',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.EntityStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ZERO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACTIVE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DELETED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DISAPPEARED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1315,
  serialized_end=1381,
)
_sym_db.RegisterEnumDescriptor(_ENTITYSTATUS)

EntityStatus = enum_type_wrapper.EnumTypeWrapper(_ENTITYSTATUS)
ZERO = 0
ACTIVE = 1
DELETED = 2
DISAPPEARED = 3



_ENUMMAPPINGDTO = _descriptor.Descriptor(
  name='EnumMappingDto',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.EnumMappingDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.EnumMappingDto.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.EnumMappingDto.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=218,
  serialized_end=263,
)


_TAGDTO_ENUMMAPPINGENTRY = _descriptor.Descriptor(
  name='EnumMappingEntry',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.EnumMappingEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.EnumMappingEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.EnumMappingEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=778,
  serialized_end=828,
)

_TAGDTO = _descriptor.Descriptor(
  name='TagDto',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='display_name', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.display_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.description', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.min_value', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.max_value', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.name', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source_id', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.source_id', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.unit', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='raw_enum_definition', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.raw_enum_definition', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.status', index=9,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='enum_mapping', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.enum_mapping', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_TAGDTO_ENUMMAPPINGENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=266,
  serialized_end=828,
)


_DATASOURCEDTO = _descriptor.Descriptor(
  name='DataSourceDto',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='identifier_type', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto.identifier_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='site_name', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto.site_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=831,
  serialized_end=959,
)


_UNITMETADATA = _descriptor.Descriptor(
  name='UnitMetadata',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='Unit', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata.Unit', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='min_value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata.min_value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='max_value', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata.max_value', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=961,
  serialized_end=1085,
)


_TAGSMETADATADTO = _descriptor.Descriptor(
  name='TagsMetadataDto',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='DataSources', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto.DataSources', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='Tags', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto.Tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='UnitTypes', full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto.UnitTypes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1088,
  serialized_end=1313,
)

_TAGDTO_ENUMMAPPINGENTRY.containing_type = _TAGDTO
_TAGDTO.fields_by_name['description'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_TAGDTO.fields_by_name['min_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_TAGDTO.fields_by_name['max_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_TAGDTO.fields_by_name['name'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_TAGDTO.fields_by_name['unit'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_TAGDTO.fields_by_name['raw_enum_definition'].message_type = google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE
_TAGDTO.fields_by_name['status'].enum_type = _ENTITYSTATUS
_TAGDTO.fields_by_name['enum_mapping'].message_type = _TAGDTO_ENUMMAPPINGENTRY
_DATASOURCEDTO.fields_by_name['identifier_type'].enum_type = Protobuf_dot_common__models__pb2._IDENTIFIERTYPES
_UNITMETADATA.fields_by_name['min_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_UNITMETADATA.fields_by_name['max_value'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_TAGSMETADATADTO.fields_by_name['DataSources'].message_type = _DATASOURCEDTO
_TAGSMETADATADTO.fields_by_name['Tags'].message_type = _TAGDTO
_TAGSMETADATADTO.fields_by_name['UnitTypes'].message_type = _UNITMETADATA
DESCRIPTOR.message_types_by_name['EnumMappingDto'] = _ENUMMAPPINGDTO
DESCRIPTOR.message_types_by_name['TagDto'] = _TAGDTO
DESCRIPTOR.message_types_by_name['DataSourceDto'] = _DATASOURCEDTO
DESCRIPTOR.message_types_by_name['UnitMetadata'] = _UNITMETADATA
DESCRIPTOR.message_types_by_name['TagsMetadataDto'] = _TAGSMETADATADTO
DESCRIPTOR.enum_types_by_name['EntityStatus'] = _ENTITYSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EnumMappingDto = _reflection.GeneratedProtocolMessageType('EnumMappingDto', (_message.Message,), {
  'DESCRIPTOR' : _ENUMMAPPINGDTO,
  '__module__' : 'tags_metadata_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.EnumMappingDto)
  })
_sym_db.RegisterMessage(EnumMappingDto)

TagDto = _reflection.GeneratedProtocolMessageType('TagDto', (_message.Message,), {

  'EnumMappingEntry' : _reflection.GeneratedProtocolMessageType('EnumMappingEntry', (_message.Message,), {
    'DESCRIPTOR' : _TAGDTO_ENUMMAPPINGENTRY,
    '__module__' : 'tags_metadata_pb2'
    # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto.EnumMappingEntry)
    })
  ,
  'DESCRIPTOR' : _TAGDTO,
  '__module__' : 'tags_metadata_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.TagDto)
  })
_sym_db.RegisterMessage(TagDto)
_sym_db.RegisterMessage(TagDto.EnumMappingEntry)

DataSourceDto = _reflection.GeneratedProtocolMessageType('DataSourceDto', (_message.Message,), {
  'DESCRIPTOR' : _DATASOURCEDTO,
  '__module__' : 'tags_metadata_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.DataSourceDto)
  })
_sym_db.RegisterMessage(DataSourceDto)

UnitMetadata = _reflection.GeneratedProtocolMessageType('UnitMetadata', (_message.Message,), {
  'DESCRIPTOR' : _UNITMETADATA,
  '__module__' : 'tags_metadata_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.UnitMetadata)
  })
_sym_db.RegisterMessage(UnitMetadata)

TagsMetadataDto = _reflection.GeneratedProtocolMessageType('TagsMetadataDto', (_message.Message,), {
  'DESCRIPTOR' : _TAGSMETADATADTO,
  '__module__' : 'tags_metadata_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataDto)
  })
_sym_db.RegisterMessage(TagsMetadataDto)


DESCRIPTOR._options = None
_TAGDTO_ENUMMAPPINGENTRY._options = None

_TAGSMETADATAGRPCSERVICE = _descriptor.ServiceDescriptor(
  name='TagsMetadataGrpcService',
  full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1384,
  serialized_end=1829,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetUnfilteredAllMetaData',
    full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService.GetUnfilteredAllMetaData',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_TAGSMETADATADTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetUnfilteredSiteMetaData',
    full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService.GetUnfilteredSiteMetaData',
    index=1,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=_TAGSMETADATADTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSiteMetaData',
    full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService.GetSiteMetaData',
    index=2,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=_TAGSMETADATADTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateTagMetaData',
    full_name='Bsii.SolarFocus.Protobuf.TagsMetadata.TagsMetadataGrpcService.UpdateTagMetaData',
    index=3,
    containing_service=None,
    input_type=_TAGDTO,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TAGSMETADATAGRPCSERVICE)

DESCRIPTOR.services_by_name['TagsMetadataGrpcService'] = _TAGSMETADATAGRPCSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
import os,sys
currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)
from datetime import datetime, timedelta
import pandas as pd
from sys import argv
from sf_virtual_data.virtual_sfv import solar_focus_sfv_lib as sofoc
from sf_virtual_data.virtual_sfv.virtual_sfv_runner import VirtualSfvRunner
from sf_virtual_data.common.api import virtual_sfv_pb2
from sf_virtual_data.common.api import solar_field_common_pb2

def main(cmd_line_args=argv[1:]) -> pd.DataFrame:
    vtCommandLineArgs, query_time_utc, margin = sofoc.parse_input(cmd_line_args)
    vt = VirtualSfvRunner(vtCommandLineArgs.metadata_file)
    res = vt.execute(vtCommandLineArgs, query_time_utc, margin)
    if not vtCommandLineArgs.debug:
        with open(vtCommandLineArgs.output_filename, 'wb') as bin_file:
            bin_text = res.SerializeToString()
            bin_file.write(bin_text)
    return res


if __name__ == "__main__":
    """
    Example of script run:  python virtual_sfv_entry_point.py --query_time_utc "2020-10-07 02:00:00" --margin "00:01:00" --debug --api-access-key "<your_api_access_key>"
    or run with IDE run-time args  ["--query_time_utc","2020-10-07 02:00:00", "--margin", "00:01:00", "--debug", "--api-access-key", "<your_api_access_key>"]

    """
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: layout.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='layout.proto',
  package='Bsii.SolarFocus.Protobuf.Layout',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0clayout.proto\x12\x1f\x42sii.SolarFocus.Protobuf.Layout\x1a\x1egoogle/protobuf/wrappers.proto\"\xef\x01\n\x0cTypedSfoList\x12:\n\x08sfo_type\x18\x01 \x01(\x0e\x32(.Bsii.SolarFocus.Protobuf.Layout.SfoType\x12\x0f\n\x03ids\x18\x02 \x03(\x05\x42\x02\x10\x01\x12\r\n\x05names\x18\x03 \x03(\t\x12\x16\n\nlocation_e\x18\x04 \x03(\x02\x42\x02\x10\x01\x12\x16\n\nlocation_n\x18\x05 \x03(\x02\x42\x02\x10\x01\x12\x16\n\nlocation_z\x18\x06 \x03(\x02\x42\x02\x10\x01\x12;\n\x05types\x18\x07 \x03(\x0e\x32(.Bsii.SolarFocus.Protobuf.Layout.SfoTypeB\x02\x10\x01\"K\n\tLayoutDto\x12>\n\x07layouts\x18\x01 \x03(\x0b\x32-.Bsii.SolarFocus.Protobuf.Layout.TypedSfoList*+\n\x07SfoType\x12\x08\n\x04ZERO\x10\x00\x12\r\n\tHELIOSTAT\x10\x01\x12\x07\n\x03\x41PU\x10\x02\x32j\n\x11LayoutGrpcService\x12U\n\tGetLayout\x12\x1c.google.protobuf.StringValue\x1a*.Bsii.SolarFocus.Protobuf.Layout.LayoutDtoB\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])

_SFOTYPE = _descriptor.EnumDescriptor(
  name='SfoType',
  full_name='Bsii.SolarFocus.Protobuf.Layout.SfoType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ZERO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HELIOSTAT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='APU', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=400,
  serialized_end=443,
)
_sym_db.RegisterEnumDescriptor(_SFOTYPE)

SfoType = enum_type_wrapper.EnumTypeWrapper(_SFOTYPE)
ZERO = 0
HELIOSTAT = 1
APU = 2



_TYPEDSFOLIST = _descriptor.Descriptor(
  name='TypedSfoList',
  full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sfo_type', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.sfo_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ids', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.ids', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='names', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.names', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location_e', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.location_e', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location_n', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.location_n', index=4,
      number=5, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location_z', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.location_z', index=5,
      number=6, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='types', full_name='Bsii.SolarFocus.Protobuf.Layout.TypedSfoList.types', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\020\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=82,
  serialized_end=321,
)


_LAYOUTDTO = _descriptor.Descriptor(
  name='LayoutDto',
  full_name='Bsii.SolarFocus.Protobuf.Layout.LayoutDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='layouts', full_name='Bsii.SolarFocus.Protobuf.Layout.LayoutDto.layouts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=323,
  serialized_end=398,
)

_TYPEDSFOLIST.fields_by_name['sfo_type'].enum_type = _SFOTYPE
_TYPEDSFOLIST.fields_by_name['types'].enum_type = _SFOTYPE
_LAYOUTDTO.fields_by_name['layouts'].message_type = _TYPEDSFOLIST
DESCRIPTOR.message_types_by_name['TypedSfoList'] = _TYPEDSFOLIST
DESCRIPTOR.message_types_by_name['LayoutDto'] = _LAYOUTDTO
DESCRIPTOR.enum_types_by_name['SfoType'] = _SFOTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TypedSfoList = _reflection.GeneratedProtocolMessageType('TypedSfoList', (_message.Message,), {
  'DESCRIPTOR' : _TYPEDSFOLIST,
  '__module__' : 'layout_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Layout.TypedSfoList)
  })
_sym_db.RegisterMessage(TypedSfoList)

LayoutDto = _reflection.GeneratedProtocolMessageType('LayoutDto', (_message.Message,), {
  'DESCRIPTOR' : _LAYOUTDTO,
  '__module__' : 'layout_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.Layout.LayoutDto)
  })
_sym_db.RegisterMessage(LayoutDto)


DESCRIPTOR._options = None
_TYPEDSFOLIST.fields_by_name['ids']._options = None
_TYPEDSFOLIST.fields_by_name['location_e']._options = None
_TYPEDSFOLIST.fields_by_name['location_n']._options = None
_TYPEDSFOLIST.fields_by_name['location_z']._options = None
_TYPEDSFOLIST.fields_by_name['types']._options = None

_LAYOUTGRPCSERVICE = _descriptor.ServiceDescriptor(
  name='LayoutGrpcService',
  full_name='Bsii.SolarFocus.Protobuf.Layout.LayoutGrpcService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=445,
  serialized_end=551,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetLayout',
    full_name='Bsii.SolarFocus.Protobuf.Layout.LayoutGrpcService.GetLayout',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=_LAYOUTDTO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_LAYOUTGRPCSERVICE)

DESCRIPTOR.services_by_name['LayoutGrpcService'] = _LAYOUTGRPCSERVICE

# @@protoc_insertion_point(module_scope)



#!/usr/bin/python3
import os,sys
currentdir = os.path.dirname(os.path.realpath(__file__))
parentdir = os.path.dirname(currentdir)
sys.path.append(parentdir)
from datetime import datetime, timedelta
from typing import List, Dict
import numpy as np
import pandas as pd
import json
import math
from sys import argv
from sf_virtual_data.virtual_trend import solar_focus_trends_lib as sofoc
from sf_virtual_data.virtual_trend.virtual_trend_runner import VirtualTrendRunner
from sf_virtual_data.common.api import common_models_pb2
from sf_virtual_data.common.api import virtual_trends_pb2
from user_calculation import calculate as user_calc
from user_calculation import plot as user_plot

def main(cmd_line_args=argv[1:]) -> pd.DataFrame:
    base_cmd_line_args, start_utc, end_utc  = sofoc.parse_input(cmd_line_args)
    vt = VirtualTrendRunner(base_cmd_line_args.metadata_file)
    res = vt.execute(base_cmd_line_args, start_utc, end_utc)
    tagTimeSeriesData = vt.df_to_tag_time_series_data_arr(res)
    if not base_cmd_line_args.debug:
        with open(base_cmd_line_args.output_filename, 'wb') as bin_file:
            bin_text = tagTimeSeriesData.SerializeToString()
            bin_file.write(bin_text)
    else:
        user_plot(res)
        
    return res


if __name__ == "__main__":
    """
    Example of script run:  python virtual_trend_entry_point.py --start-utc "2020-10-07 02:00:00" --end-utc "2020-10-07 16:00:00" --debug --api-access-key "<your_api_access_key>"
    or run with IDE run-time args  ["--start-utc","2020-10-07 02:00:00", "--end-utc", "2020-10-07 16:00:00", "--debug", "--api-access-key", "<your_api_access_key>"]
    """
    main()



#!/usr/bin/python3
import pandas as pd
from datetime import datetime, timedelta

def calculate(data: pd.DataFrame, start_utc: datetime, end_utc: datetime) -> pd.DataFrame:
    """
    User main code to calculate new virtual sfv based on other sfv's

    Parameters
    ---------
        pandas DataFrame. columns are based on define_inputs()

    Returns
    -------
        pd.DataFrame
        pandas DataFrame. column must match the defined output as implemented in define_outputs(value_column   """

    ################# User Code #########################

    # This returns value_column tag "HlstShortStatus.currentState" from Mongo collection
    # res = samples.read_from_mongo_closest_time(query_time_utc, margin,"_id","Data", "HlstShortStatus.currentState")

    # This returns output for 1 tag "AverageDnrWattPerSquareMetre" from Sql Server (CALS table)
    res = read_from_sql_server(start_utc, end_utc)

    ################# End of User Code ###################
    return res


def read_from_sql_server(query_time_utc: datetime, margin: timedelta) -> pd.DataFrame:
    from .sql_reader import SqlReader
    date_format_to_sql_server = "%Y-%m-%d"
    query = ''' SELECT * FROM (SELECT DISTINCT [HeliostatId], [AverageDnrWattPerSquareMetre], [MeasureTime], 
                    RANK() OVER (PARTITION BY [HeliostatId] ORDER BY MeasureTime DESC) dest_rank
                    FROM [NoorPA_SfincsData].[CMS_CALS].[Measurements]) T
	            where T.dest_rank = 1
    '''

    reader = SqlReader(SqlReader.create_sql_server_trusted_connection_string("site-sql-data", "NoorPA_SfincsData"))
    date_format_from_sql_server = "%d/%m/%Y"
    df = reader.get_df_from_db(query, "AverageDnrWattPerSquareMetre","HeliostatId", date_format=date_format_from_sql_server)
    return df


def read_from_mongo_closest_time(query_time_utc: datetime, margin: timedelta, timestamp_field:str, value_field:str, collection_name: str):
    from .mongo_reader import MongoReader
    reader = MongoReader("ConnectionString")
     # Make a query to the specific DB and Collection
    reader.get_collection(collection_name)
    pipeline = [{'$match' :{'{0}'.format(timestamp_field):{'$gte': query_time_utc-margin,'$lte': query_time_utc+margin}}},
                { '$addFields' : { 'distance': {'$abs': { '$subtract': [query_time_utc, '${0}'.format(timestamp_field)]}}}},
                { '$sort' : { 'distance' : 1 } },
                { '$limit' : 1 }
                ]  
    cursor = reader.get_collection(collection_name).aggregate(pipeline)
    # Expand the cursor and construct the DataFrame
    data = list(cursor)
    doc = data[0]
    df =  pd.DataFrame()
    df[value_field] = doc[value_field]
    return df


#!/usr/bin/python3
import argparse
import json
import os
from common.api import virtual_tables_pb2_grpc
from common.api import virtual_tables_pb2
from common.api import solar_field_common_pb2
from common import utils
from common import models
import grpc
from datetime import datetime, timedelta
from common.models import VTCommandLineArgsBase
TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
import pandas as pd

def get_data(query: virtual_tables_pb2.VirtualTablesQuery, args: models.VTCommandLineArgsBase) -> virtual_tables_pb2.VirtualTablesInputsResult:
    channel = utils.get_grpc_channel_according_to_args(args)
    client = virtual_tables_pb2_grpc.VirtualTablesGrpcServiceStub(channel)
    res = client.GetData(query, metadata=[('x-api-key', f'{args.api_access_key}')])
    return res




def parse_input(cmd_line_args) -> (models.VTCommandLineArgsBase, datetime, timedelta, datetime, datetime):
    (base_cmd_line_args, extra_args) = utils.parse_input(cmd_line_args)
    parser = argparse.ArgumentParser(
        description='Input parameters to run virtual tables')
    parser.add_argument('--sfv-query-time-utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='sfv_query_time_utc',
                        help='The sfv query time in UTC, format YYYY-MM-DD hh:mm:ss')
    parser.add_argument('--sfv-margin', action='store', required='--metadata' not in cmd_line_args,
                        dest='sfv_margin',
                        help='The sfv query margin time of the query in UTC, format hh:mm:ss')
    parser.add_argument('--trends-start-utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='trends_start_utc',
                        help='Start time of the trends query in UTC, format YYYY-MM-DD hh:mm:ss')
    parser.add_argument('--trends-end-utc', action='store', required='--metadata' not in cmd_line_args,
                        dest='trends_end_utc',
                        help='End time of the trends query in UTC, format YYYY-MM-DD hh:mm:ss')
    args = parser.parse_args(extra_args)
    if args.sfv_query_time_utc is not None and args.sfv_margin is not None:
        sfv_query_time_utc = datetime.strptime(args.sfv_query_time_utc, TIME_FORMAT)
        sfv_margin = utils.parse_time(args.sfv_margin)
    else:
        query_time_utc = None
        margin = None
    if args.trends_start_utc is not None and args.trends_end_utc is not None:
        trends_start_utc = datetime.strptime(args.trends_start_utc, TIME_FORMAT)
        trends_end_utc = datetime.strptime(args.trends_end_utc, TIME_FORMAT)
    else:
        trends_start_utc = None
        trends_end_utc = None
    return base_cmd_line_args, sfv_query_time_utc, sfv_margin, trends_start_utc, trends_end_utc



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: solar-field-common.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from . import common_models_pb2 as Protobuf_dot_common__models__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='solar-field-common.proto',
  package='Bsii.SolarFocus.Protobuf.SolarField',
  syntax='proto3',
  serialized_options=b'\252\002\034Bsii.SolarFocus.Lib.Protobuf',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x18solar-field-common.proto\x12#Bsii.SolarFocus.Protobuf.SolarField\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1cProtobuf/common-models.proto\"\xac\x01\n\x07TagData\x12<\n\x06tag_id\x18\x01 \x01(\x0b\x32,.Bsii.SolarFocus.Protobuf.TagsIdentifierBase\x12/\n\x0bupdate_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\rnullable_data\x18\x03 \x03(\x0b\x32\x1b.google.protobuf.FloatValue\"\x8f\x01\n\rSolarFieldDto\x12:\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32,.Bsii.SolarFocus.Protobuf.SolarField.TagData\x12\x42\n\x10problematic_tags\x18\x02 \x03(\x0b\x32(.Bsii.SolarFocus.Protobuf.ProblematicTag*\x8e\x01\n\x14SfvCalculationMethod\x12\x10\n\x0c\x41NOTHER_ZERO\x10\x00\x12\x12\n\x0e\x43LOSEST_SAMPLE\x10\x01\x12\x1c\n\x18\x41VERAGE_OVER_SAMPLE_FREQ\x10\x02\x12\x18\n\x14MAX_OVER_SAMPLE_FREQ\x10\x03\x12\x18\n\x14MIN_OVER_SAMPLE_FREQ\x10\x04\x42\x1f\xaa\x02\x1c\x42sii.SolarFocus.Lib.Protobufb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,Protobuf_dot_common__models__pb2.DESCRIPTOR,])

_SFVCALCULATIONMETHOD = _descriptor.EnumDescriptor(
  name='SfvCalculationMethod',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.SfvCalculationMethod',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ANOTHER_ZERO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CLOSEST_SAMPLE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='AVERAGE_OVER_SAMPLE_FREQ', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAX_OVER_SAMPLE_FREQ', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MIN_OVER_SAMPLE_FREQ', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=482,
  serialized_end=624,
)
_sym_db.RegisterEnumDescriptor(_SFVCALCULATIONMETHOD)

SfvCalculationMethod = enum_type_wrapper.EnumTypeWrapper(_SFVCALCULATIONMETHOD)
ANOTHER_ZERO = 0
CLOSEST_SAMPLE = 1
AVERAGE_OVER_SAMPLE_FREQ = 2
MAX_OVER_SAMPLE_FREQ = 3
MIN_OVER_SAMPLE_FREQ = 4



_TAGDATA = _descriptor.Descriptor(
  name='TagData',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.TagData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_id', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagData.tag_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagData.update_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='nullable_data', full_name='Bsii.SolarFocus.Protobuf.SolarField.TagData.nullable_data', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=161,
  serialized_end=333,
)


_SOLARFIELDDTO = _descriptor.Descriptor(
  name='SolarFieldDto',
  full_name='Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto.data', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='problematic_tags', full_name='Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto.problematic_tags', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=479,
)

_TAGDATA.fields_by_name['tag_id'].message_type = Protobuf_dot_common__models__pb2._TAGSIDENTIFIERBASE
_TAGDATA.fields_by_name['update_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_TAGDATA.fields_by_name['nullable_data'].message_type = google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE
_SOLARFIELDDTO.fields_by_name['data'].message_type = _TAGDATA
_SOLARFIELDDTO.fields_by_name['problematic_tags'].message_type = Protobuf_dot_common__models__pb2._PROBLEMATICTAG
DESCRIPTOR.message_types_by_name['TagData'] = _TAGDATA
DESCRIPTOR.message_types_by_name['SolarFieldDto'] = _SOLARFIELDDTO
DESCRIPTOR.enum_types_by_name['SfvCalculationMethod'] = _SFVCALCULATIONMETHOD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TagData = _reflection.GeneratedProtocolMessageType('TagData', (_message.Message,), {
  'DESCRIPTOR' : _TAGDATA,
  '__module__' : 'solar_field_common_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.TagData)
  })
_sym_db.RegisterMessage(TagData)

SolarFieldDto = _reflection.GeneratedProtocolMessageType('SolarFieldDto', (_message.Message,), {
  'DESCRIPTOR' : _SOLARFIELDDTO,
  '__module__' : 'solar_field_common_pb2'
  # @@protoc_insertion_point(class_scope:Bsii.SolarFocus.Protobuf.SolarField.SolarFieldDto)
  })
_sym_db.RegisterMessage(SolarFieldDto)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
